import { AxiosRequestConfig, AxiosError, AxiosResponse} from 'axios';
import axios from 'axios';
import _ from 'lodash';

import messageBoxService from './message-box.service';

const notRequireInterceptorErrorUrl = [
  '/otp/verify',
  '/eService/authenticate',
  '/login/validate/get_img_code',
  '/login/validate/verify_img_code',
];

export const http = axios.create({
  baseURL: process.env.BASE_URL,
  withCredentials: true
});

let apiNumber = 0;

http.interceptors.request.use(
  (config: AxiosRequestConfig) => {
    apiNumber += 1;
    loadingStart();
    addHttpHeader(config);
    return config;
  }
);

http.interceptors.response.use(
  (response: AxiosResponse) => {
    apiNumber -= 1;
    if (apiNumber === 0) {
      loadingFinish();
    }
    return response;
  },
  (error: AxiosError) => {
    apiNumber -= 1;
    if (apiNumber === 0) {
      loadingFinish();
    }
    showErrorMessageBox(error)
    return Promise.reject(error);
  }
);

function addHttpHeader(config: AxiosRequestConfig): void {
  config.headers = {
    Authorization: 'Bearer ' + localStorage.getItem('id_token')
  }
}

function loadingStart(): void {
  setTimeout(() => {
    window.$nuxt.$loading.start();
  }, 0);
};

function loadingFinish(): void {
  setTimeout(() => {
    window.$nuxt.$loading.finish();
  }, 0);
};

function showErrorMessageBox(error: any): void {
  // console.log('error', error, error.response);
  setTimeout(() => {
    if (!_.includes(notRequireInterceptorErrorUrl, error.config.url)) {
      switch (error.response.status) {
        case 401:
          Promise.all([messageBoxService.showErrorMessage('登入逾時'), window.$nuxt.$store.dispatch('localStorage/actionStorageClear')]).then(() => {
            _.isEqual(window.$nuxt.$route.name, 'index') ? location.reload() : window.$nuxt.$router.push('/');
          });
          break;

        default:
          messageBoxService.showErrorMessage('', error);
          break;
      }
    }
  }, 0)
};
